@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2016 The PHP Group                                |
+  | Copyright (c) 1997-2015 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -22,34 +22,30 @@
 
 void php_filter_callback(PHP_INPUT_FILTER_PARAM_DECL)
 {
-	zval *retval_ptr;
-	zval ***args;
+	zval retval;
+	zval *args;
 	int status;
 
-	if (!option_array || !zend_is_callable(option_array, IS_CALLABLE_CHECK_NO_ACCESS, NULL TSRMLS_CC)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "First argument is expected to be a valid callback");
-		zval_dtor(value);
-		Z_TYPE_P(value) = IS_NULL;
+	if (!option_array || !zend_is_callable(option_array, IS_CALLABLE_CHECK_NO_ACCESS, NULL)) {
+		php_error_docref(NULL, E_WARNING, "First argument is expected to be a valid callback");
+		zval_ptr_dtor(value);
+		ZVAL_NULL(value);
 		return;
 	}
 
-	args = safe_emalloc(sizeof(zval **), 1, 0);
-	args[0] = &value;
-	
-	status = call_user_function_ex(EG(function_table), NULL, option_array, &retval_ptr, 1, args, 0, NULL TSRMLS_CC);
+	args = safe_emalloc(sizeof(zval), 1, 0);
+	ZVAL_COPY(&args[0], value);
+	status = call_user_function_ex(EG(function_table), NULL, option_array, &retval, 1, args, 0, NULL);
 
-	if (status == SUCCESS && retval_ptr != NULL) {
-		if (retval_ptr != value) {
-			zval_dtor(value);
-			COPY_PZVAL_TO_ZVAL(*value, retval_ptr);
-		} else {
-			zval_ptr_dtor(&retval_ptr);
-		}
+	if (status == SUCCESS && !Z_ISUNDEF(retval)) {
+		zval_ptr_dtor(value);
+		ZVAL_COPY_VALUE(value, &retval);
 	} else {
-		zval_dtor(value);
-		Z_TYPE_P(value) = IS_NULL;
+		zval_ptr_dtor(value);
+		ZVAL_NULL(value);
 	}
 
+	zval_ptr_dtor(&args[0]);
 	efree(args);
 }
 
