@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -65,7 +65,7 @@
 #include <sys/select.h>
 #endif
 
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 #include <openssl/ssl.h>
 #endif
 
@@ -100,16 +100,16 @@ static int		ftp_getresp(ftpbuf_t *ftp);
 static int		ftp_type(ftpbuf_t *ftp, ftptype_t type);
 
 /* opens up a data stream */
-static databuf_t*	ftp_getdata(ftpbuf_t *ftp TSRMLS_DC);
+static databuf_t*	ftp_getdata(ftpbuf_t *ftp);
 
 /* accepts the data connection, returns updated data buffer */
-static databuf_t*	data_accept(databuf_t *data, ftpbuf_t *ftp TSRMLS_DC);
+static databuf_t*	data_accept(databuf_t *data, ftpbuf_t *ftp);
 
 /* closes the data connection, returns NULL */
 static databuf_t*	data_close(ftpbuf_t *ftp, databuf_t *data);
 
 /* generic file lister */
-static char**		ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC);
+static char**		ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path);
 
 /* IP and port conversion box */
 union ipbox {
@@ -121,7 +121,7 @@ union ipbox {
 /* {{{ ftp_open
  */
 ftpbuf_t*
-ftp_open(const char *host, short port, long timeout_sec TSRMLS_DC)
+ftp_open(const char *host, short port, zend_long timeout_sec)
 {
 	ftpbuf_t		*ftp;
 	socklen_t		 size;
@@ -136,7 +136,7 @@ ftp_open(const char *host, short port, long timeout_sec TSRMLS_DC)
 
 	ftp->fd = php_network_connect_socket_to_host(host,
 			(unsigned short) (port ? port : 21), SOCK_STREAM,
-			0, &tv, NULL, NULL, NULL, 0 TSRMLS_CC);
+			0, &tv, NULL, NULL, NULL, 0, STREAM_SOCKOP_NONE);
 	if (ftp->fd == -1) {
 		goto bail;
 	}
@@ -148,7 +148,7 @@ ftp_open(const char *host, short port, long timeout_sec TSRMLS_DC)
 	size = sizeof(ftp->localaddr);
 	memset(&ftp->localaddr, 0, size);
 	if (getsockname(ftp->fd, (struct sockaddr*) &ftp->localaddr, &size) != 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "getsockname failed: %s (%d)", strerror(errno), errno);
+		php_error_docref(NULL, E_WARNING, "getsockname failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
@@ -179,11 +179,10 @@ ftp_close(ftpbuf_t *ftp)
 		data_close(ftp, ftp->data);
 	}
 	if (ftp->stream && ftp->closestream) {
-		TSRMLS_FETCH();
-		php_stream_close(ftp->stream);
+			php_stream_close(ftp->stream);
 	}
 	if (ftp->fd != -1) {
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 		if (ftp->ssl_active) {
 			SSL_shutdown(ftp->ssl_handle);
 			SSL_free(ftp->ssl_handle);
@@ -244,17 +243,19 @@ ftp_quit(ftpbuf_t *ftp)
 /* {{{ ftp_login
  */
 int
-ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
+ftp_login(ftpbuf_t *ftp, const char *user, const char *pass)
 {
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	SSL_CTX	*ctx = NULL;
 	long ssl_ctx_options = SSL_OP_ALL;
+	int err, res;
+	zend_bool retry;
 #endif
 	if (ftp == NULL) {
 		return 0;
 	}
 
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	if (ftp->use_ssl && !ftp->ssl_active) {
 		if (!ftp_putcmd(ftp, "AUTH", "TLS")) {
 			return 0;
@@ -281,7 +282,7 @@ ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
 
 		ctx = SSL_CTX_new(SSLv23_client_method());
 		if (ctx == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL context");
+			php_error_docref(NULL, E_WARNING, "failed to create the SSL context");
 			return 0;
 		}
 
@@ -290,24 +291,52 @@ ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
 #endif
 		SSL_CTX_set_options(ctx, ssl_ctx_options);
 
-		/* allow SSL to re-use sessions */
-		SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_BOTH);
-
 		ftp->ssl_handle = SSL_new(ctx);
 		if (ftp->ssl_handle == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL handle");
+			php_error_docref(NULL, E_WARNING, "failed to create the SSL handle");
 			SSL_CTX_free(ctx);
 			return 0;
 		}
 
 		SSL_set_fd(ftp->ssl_handle, ftp->fd);
 
-		if (SSL_connect(ftp->ssl_handle) <= 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS handshake failed");
-			SSL_shutdown(ftp->ssl_handle);
-			SSL_free(ftp->ssl_handle);
-			return 0;
-		}
+		do {
+			res = SSL_connect(ftp->ssl_handle);
+			err = SSL_get_error(ftp->ssl_handle, res);
+
+			/* TODO check if handling other error codes would make sense */
+			switch (err) {
+				case SSL_ERROR_NONE:
+					retry = 0;
+					break;
+
+				case SSL_ERROR_ZERO_RETURN:
+					retry = 0;
+					SSL_shutdown(ftp->ssl_handle);
+					break;
+
+				case SSL_ERROR_WANT_READ:
+				case SSL_ERROR_WANT_WRITE: {
+						php_pollfd p;
+						int i;
+
+						p.fd = ftp->fd;
+						p.events = (err == SSL_ERROR_WANT_READ) ? (POLLIN|POLLPRI) : POLLOUT;
+						p.revents = 0;
+
+						i = php_poll2(&p, 1, 300);
+
+						retry = i > 0;
+					}
+					break;
+
+				default:
+					php_error_docref(NULL, E_WARNING, "SSL/TLS handshake failed");
+					SSL_shutdown(ftp->ssl_handle);
+					SSL_free(ftp->ssl_handle);
+					return 0;
+			}
+		} while (retry);
 
 		ftp->ssl_active = 1;
 
@@ -482,7 +511,7 @@ ftp_raw(ftpbuf_t *ftp, const char *cmd, zval *return_value)
 	}
 	array_init(return_value);
 	while (ftp_readline(ftp)) {
-		add_next_index_string(return_value, ftp->inbuf, 1);
+		add_next_index_string(return_value, ftp->inbuf);
 		if (isdigit(ftp->inbuf[0]) && isdigit(ftp->inbuf[1]) && isdigit(ftp->inbuf[2]) && ftp->inbuf[3] == ' ') {
 			return;
 		}
@@ -540,10 +569,11 @@ ftp_cdup(ftpbuf_t *ftp)
 
 /* {{{ ftp_mkdir
  */
-char*
+zend_string*
 ftp_mkdir(ftpbuf_t *ftp, const char *dir)
 {
 	char *mkd, *end;
+	zend_string *ret;
 
 	if (ftp == NULL) {
 		return NULL;
@@ -556,17 +586,16 @@ ftp_mkdir(ftpbuf_t *ftp, const char *dir)
 	}
 	/* copy out the dir from response */
 	if ((mkd = strchr(ftp->inbuf, '"')) == NULL) {
-		mkd = estrdup(dir);
-		return mkd;
+		return zend_string_init(dir, strlen(dir), 0);
 	}
 	if ((end = strrchr(++mkd, '"')) == NULL) {
 		return NULL;
 	}
 	*end = 0;
-	mkd = estrdup(mkd);
+	ret = zend_string_init(mkd, end - mkd, 0);
 	*end = '"';
 
-	return mkd;
+	return ret;
 }
 /* }}} */
 
@@ -619,7 +648,7 @@ ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filenam
 /* {{{ ftp_alloc
  */
 int
-ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
+ftp_alloc(ftpbuf_t *ftp, const zend_long size, zend_string **response)
 {
 	char buffer[64];
 
@@ -627,7 +656,7 @@ ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
 		return 0;
 	}
 
-	snprintf(buffer, sizeof(buffer) - 1, "%ld", size);
+	snprintf(buffer, sizeof(buffer) - 1, ZEND_LONG_FMT, size);
 
 	if (!ftp_putcmd(ftp, "ALLO", buffer)) {
 		return 0;
@@ -638,7 +667,7 @@ ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
 	}
 
 	if (response) {
-		*response = estrdup(ftp->inbuf);
+		*response = zend_string_init(ftp->inbuf, strlen(ftp->inbuf), 0);
 	}
 
 	if (ftp->resp < 200 || ftp->resp >= 300) {
@@ -652,18 +681,18 @@ ftp_alloc(ftpbuf_t *ftp, const long size, char **response)
 /* {{{ ftp_nlist
  */
 char**
-ftp_nlist(ftpbuf_t *ftp, const char *path TSRMLS_DC)
+ftp_nlist(ftpbuf_t *ftp, const char *path)
 {
-	return ftp_genlist(ftp, "NLST", path TSRMLS_CC);
+	return ftp_genlist(ftp, "NLST", path);
 }
 /* }}} */
 
 /* {{{ ftp_list
  */
 char**
-ftp_list(ftpbuf_t *ftp, const char *path, int recursive TSRMLS_DC)
+ftp_list(ftpbuf_t *ftp, const char *path, int recursive)
 {
-	return ftp_genlist(ftp, ((recursive) ? "LIST -R" : "LIST"), path TSRMLS_CC);
+	return ftp_genlist(ftp, ((recursive) ? "LIST -R" : "LIST"), path);
 }
 /* }}} */
 
@@ -725,11 +754,10 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 	memset(&ftp->pasvaddr, 0, n);
 	sa = (struct sockaddr *) &ftp->pasvaddr;
 
+#if HAVE_IPV6
 	if (getpeername(ftp->fd, sa, &n) < 0) {
 		return 0;
 	}
-
-#if HAVE_IPV6
 	if (sa->sa_family == AF_INET6) {
 		struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) sa;
 		char *endptr, delimiter;
@@ -782,9 +810,8 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 		ipbox.c[n] = (unsigned char) b[n];
 	}
 	sin = (struct sockaddr_in *) sa;
-	if (ftp->usepasvaddress) {
-		sin->sin_addr = ipbox.ia[0];
-	}
+	sin->sin_family = AF_INET;
+	sin->sin_addr = ipbox.ia[0];
 	sin->sin_port = ipbox.s[2];
 
 	ftp->pasv = 2;
@@ -796,7 +823,7 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 /* {{{ ftp_get
  */
 int
-ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, long resumepos TSRMLS_DC)
+ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, zend_long resumepos)
 {
 	databuf_t		*data = NULL;
 	size_t			rcvd;
@@ -809,14 +836,14 @@ ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type,
 		goto bail;
 	}
 
-	if ((data = ftp_getdata(ftp TSRMLS_CC)) == NULL) {
+	if ((data = ftp_getdata(ftp)) == NULL) {
 		goto bail;
 	}
 
 	ftp->data = data;
 
 	if (resumepos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", resumepos);
+		snprintf(arg, sizeof(arg), ZEND_LONG_FMT, resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -832,7 +859,7 @@ ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type,
 		goto bail;
 	}
 
-	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
+	if ((data = data_accept(data, ftp)) == NULL) {
 		goto bail;
 	}
 
@@ -888,10 +915,10 @@ bail:
 /* {{{ ftp_put
  */
 int
-ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, long startpos TSRMLS_DC)
+ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, zend_long startpos)
 {
 	databuf_t		*data = NULL;
-	long			size;
+	zend_long			size;
 	char			*ptr;
 	int			ch;
 	char			arg[11];
@@ -902,13 +929,13 @@ ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, l
 	if (!ftp_type(ftp, type)) {
 		goto bail;
 	}
-	if ((data = ftp_getdata(ftp TSRMLS_CC)) == NULL) {
+	if ((data = ftp_getdata(ftp)) == NULL) {
 		goto bail;
 	}
 	ftp->data = data;
 
 	if (startpos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", startpos);
+		snprintf(arg, sizeof(arg), ZEND_LONG_FMT, startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -923,7 +950,7 @@ ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, l
 	if (!ftp_getresp(ftp) || (ftp->resp != 150 && ftp->resp != 125)) {
 		goto bail;
 	}
-	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
+	if ((data = data_accept(data, ftp)) == NULL) {
 		goto bail;
 	}
 
@@ -965,7 +992,7 @@ bail:
 
 /* {{{ ftp_size
  */
-long
+zend_long
 ftp_size(ftpbuf_t *ftp, const char *path)
 {
 	if (ftp == NULL) {
@@ -1232,16 +1259,26 @@ ftp_getresp(ftpbuf_t *ftp)
 int
 my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
-	long		size, sent;
+	zend_long		size, sent;
     int         n;
+#ifdef HAVE_FTP_SSL
+	int err;
+	zend_bool retry = 0;
+	SSL *handle = NULL;
+	php_socket_t fd;
+#endif
+
 
 	size = len;
 	while (size) {
 		n = php_pollfd_for_ms(s, POLLOUT, ftp->timeout_sec * 1000);
 
 		if (n < 1) {
-
-#if !defined(PHP_WIN32) && !(defined(NETWARE) && defined(USE_WINSOCK))
+#ifdef PHP_WIN32
+			if (n == 0) {
+				_set_errno(ETIMEDOUT);
+			}
+#elif !(defined(NETWARE) && defined(USE_WINSOCK))
 			if (n == 0) {
 				errno = ETIMEDOUT;
 			}
@@ -1249,15 +1286,54 @@ my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 			return -1;
 		}
 
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 		if (ftp->use_ssl && ftp->fd == s && ftp->ssl_active) {
-			sent = SSL_write(ftp->ssl_handle, buf, size);
+			handle = ftp->ssl_handle;
+			fd = ftp->fd;
 		} else if (ftp->use_ssl && ftp->fd != s && ftp->use_ssl_for_data && ftp->data->ssl_active) {
-			sent = SSL_write(ftp->data->ssl_handle, buf, size);
+			handle = ftp->data->ssl_handle;
+			fd = ftp->data->fd;
+		}
+
+		if (handle) {
+			do {
+				sent = SSL_write(handle, buf, size);
+				err = SSL_get_error(handle, sent);
+
+				switch (err) {
+					case SSL_ERROR_NONE:
+						retry = 0;
+						break;
+
+					case SSL_ERROR_ZERO_RETURN:
+						retry = 0;
+						SSL_shutdown(handle);
+						break;
+
+					case SSL_ERROR_WANT_READ:
+					case SSL_ERROR_WANT_CONNECT: {
+							php_pollfd p;
+							int i;
+
+							p.fd = fd;
+							p.events = POLLOUT;
+							p.revents = 0;
+
+							i = php_poll2(&p, 1, 300);
+
+							retry = i > 0;
+						}
+						break;
+
+					default:
+						php_error_docref(NULL, E_WARNING, "SSL write failed");
+						return -1;
+				}
+			} while (retry);
 		} else {
 #endif
 			sent = send(s, buf, size, 0);
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 		}
 #endif
 		if (sent == -1) {
@@ -1278,10 +1354,20 @@ int
 my_recv(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
 	int		n, nr_bytes;
+#ifdef HAVE_FTP_SSL
+	int err;
+	zend_bool retry = 0;
+	SSL *handle = NULL;
+	php_socket_t fd;
+#endif
 
 	n = php_pollfd_for_ms(s, PHP_POLLREADABLE, ftp->timeout_sec * 1000);
 	if (n < 1) {
-#if !defined(PHP_WIN32) && !(defined(NETWARE) && defined(USE_WINSOCK))
+#ifdef PHP_WIN32
+		if (n == 0) {
+			_set_errno(ETIMEDOUT);
+		}
+#elif !(defined(NETWARE) && defined(USE_WINSOCK))
 		if (n == 0) {
 			errno = ETIMEDOUT;
 		}
@@ -1289,15 +1375,54 @@ my_recv(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 		return -1;
 	}
 
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	if (ftp->use_ssl && ftp->fd == s && ftp->ssl_active) {
-		nr_bytes = SSL_read(ftp->ssl_handle, buf, len);
+		handle = ftp->ssl_handle;
+		fd = ftp->fd;
 	} else if (ftp->use_ssl && ftp->fd != s && ftp->use_ssl_for_data && ftp->data->ssl_active) {
-		nr_bytes = SSL_read(ftp->data->ssl_handle, buf, len);
+		handle = ftp->data->ssl_handle;
+		fd = ftp->data->fd;
+	}
+
+	if (handle) {
+		do {
+			nr_bytes = SSL_read(handle, buf, len);
+			err = SSL_get_error(handle, nr_bytes);
+
+			switch (err) {
+				case SSL_ERROR_NONE:
+					retry = 0;
+					break;
+
+				case SSL_ERROR_ZERO_RETURN:
+					retry = 0;
+					SSL_shutdown(handle);
+					break;
+
+				case SSL_ERROR_WANT_READ:
+				case SSL_ERROR_WANT_CONNECT: {
+						php_pollfd p;
+						int i;
+
+						p.fd = fd;
+						p.events = POLLIN|POLLPRI;
+						p.revents = 0;
+
+						i = php_poll2(&p, 1, 300);
+
+						retry = i > 0;
+					}
+					break;
+
+				default:
+					php_error_docref(NULL, E_WARNING, "SSL read failed");
+					return -1;
+			}
+		} while (retry);
 	} else {
 #endif
 		nr_bytes = recv(s, buf, len, 0);
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	}
 #endif
 	return (nr_bytes);
@@ -1313,7 +1438,11 @@ data_available(ftpbuf_t *ftp, php_socket_t s)
 
 	n = php_pollfd_for_ms(s, PHP_POLLREADABLE, 1000);
 	if (n < 1) {
-#if !defined(PHP_WIN32) && !(defined(NETWARE) && defined(USE_WINSOCK))
+#ifdef PHP_WIN32
+		if (n == 0) {
+			_set_errno(ETIMEDOUT);
+		}
+#elif !(defined(NETWARE) && defined(USE_WINSOCK))
 		if (n == 0) {
 			errno = ETIMEDOUT;
 		}
@@ -1333,7 +1462,11 @@ data_writeable(ftpbuf_t *ftp, php_socket_t s)
 
 	n = php_pollfd_for_ms(s, POLLOUT, 1000);
 	if (n < 1) {
-#ifndef PHP_WIN32
+#ifdef PHP_WIN32
+		if (n == 0) {
+			_set_errno(ETIMEDOUT);
+		}
+#elif !(defined(NETWARE) && defined(USE_WINSOCK))
 		if (n == 0) {
 			errno = ETIMEDOUT;
 		}
@@ -1354,7 +1487,11 @@ my_accept(ftpbuf_t *ftp, php_socket_t s, struct sockaddr *addr, socklen_t *addrl
 
 	n = php_pollfd_for_ms(s, PHP_POLLREADABLE, ftp->timeout_sec * 1000);
 	if (n < 1) {
-#if !defined(PHP_WIN32) && !(defined(NETWARE) && defined(USE_WINSOCK))
+#ifdef PHP_WIN32
+		if (n == 0) {
+			_set_errno(ETIMEDOUT);
+		}
+#elif !(defined(NETWARE) && defined(USE_WINSOCK))
 		if (n == 0) {
 			errno = ETIMEDOUT;
 		}
@@ -1369,7 +1506,7 @@ my_accept(ftpbuf_t *ftp, php_socket_t s, struct sockaddr *addr, socklen_t *addrl
 /* {{{ ftp_getdata
  */
 databuf_t*
-ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
+ftp_getdata(ftpbuf_t *ftp)
 {
 	int			fd = -1;
 	databuf_t		*data;
@@ -1394,7 +1531,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 	sa = (struct sockaddr *) &ftp->localaddr;
 	/* bind/listen */
 	if ((fd = socket(sa->sa_family, SOCK_STREAM, 0)) == SOCK_ERR) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "socket() failed: %s (%d)", strerror(errno), errno);
+		php_error_docref(NULL, E_WARNING, "socket() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
@@ -1409,7 +1546,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 		tv.tv_sec = ftp->timeout_sec;
 		tv.tv_usec = 0;
 		if (php_connect_nonb(fd, (struct sockaddr*) &ftp->pasvaddr, size, &tv) == -1) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_connect_nonb() failed: %s (%d)", strerror(errno), errno);
+			php_error_docref(NULL, E_WARNING, "php_connect_nonb() failed: %s (%d)", strerror(errno), errno);
 			goto bail;
 		}
 
@@ -1427,17 +1564,17 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 	size = php_sockaddr_size(&addr);
 
 	if (bind(fd, (struct sockaddr*) &addr, size) != 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "bind() failed: %s (%d)", strerror(errno), errno);
+		php_error_docref(NULL, E_WARNING, "bind() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
 	if (getsockname(fd, (struct sockaddr*) &addr, &size) != 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "getsockname() failed: %s (%d)", strerror(errno), errno);
+		php_error_docref(NULL, E_WARNING, "getsockname() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
 	if (listen(fd, 5) != 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "listen() failed: %s (%d)", strerror(errno), errno);
+		php_error_docref(NULL, E_WARNING, "listen() failed: %s (%d)", strerror(errno), errno);
 		goto bail;
 	}
 
@@ -1491,15 +1628,16 @@ bail:
 /* {{{ data_accept
  */
 databuf_t*
-data_accept(databuf_t *data, ftpbuf_t *ftp TSRMLS_DC)
+data_accept(databuf_t *data, ftpbuf_t *ftp)
 {
 	php_sockaddr_storage addr;
 	socklen_t			size;
 
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	SSL_CTX		*ctx;
-	SSL_SESSION *session;
-	int result;
+	zend_long ssl_ctx_options = SSL_OP_ALL;
+	int err, res;
+	zend_bool retry;
 #endif
 
 	if (data->fd != -1) {
@@ -1516,50 +1654,71 @@ data_accept(databuf_t *data, ftpbuf_t *ftp TSRMLS_DC)
 	}
 
 data_accepted:
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 
 	/* now enable ssl if we need to */
 	if (ftp->use_ssl && ftp->use_ssl_for_data) {
-		ctx = SSL_get_SSL_CTX(ftp->ssl_handle);
+		ctx = SSL_CTX_new(SSLv23_client_method());
 		if (ctx == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: failed to retreive the existing SSL context");
+			php_error_docref(NULL, E_WARNING, "data_accept: failed to create the SSL context");
 			return 0;
 		}
 
+#if OPENSSL_VERSION_NUMBER >= 0x0090605fL
+		ssl_ctx_options &= ~SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS;
+#endif
+		SSL_CTX_set_options(ctx, ssl_ctx_options);
+
 		data->ssl_handle = SSL_new(ctx);
 		if (data->ssl_handle == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: failed to create the SSL handle");
+			php_error_docref(NULL, E_WARNING, "data_accept: failed to create the SSL handle");
+			SSL_CTX_free(ctx);
 			return 0;
 		}
 
+
 		SSL_set_fd(data->ssl_handle, data->fd);
 
 		if (ftp->old_ssl) {
 			SSL_copy_session_id(data->ssl_handle, ftp->ssl_handle);
 		}
 
-		/* get the session from the control connection so we can re-use it */
-		session = SSL_get_session(ftp->ssl_handle);
-		if (session == NULL) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: failed to retreive the existing SSL session");
-			SSL_free(data->ssl_handle);
-			return 0;
-		}
+		do {
+			res = SSL_connect(data->ssl_handle);
+			err = SSL_get_error(data->ssl_handle, res);
 
-		/* and set it on the data connection */
-		result = SSL_set_session(data->ssl_handle, session);
-		if (result == 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: failed to set the existing SSL session");
-			SSL_free(data->ssl_handle);
-			return 0;
-		}
+			switch (err) {
+				case SSL_ERROR_NONE:
+					retry = 0;
+					break;
 
-		if (SSL_connect(data->ssl_handle) <= 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "data_accept: SSL/TLS handshake failed");
-			SSL_shutdown(data->ssl_handle);
-			SSL_free(data->ssl_handle);
-			return 0;
-		}
+				case SSL_ERROR_ZERO_RETURN:
+					retry = 0;
+					SSL_shutdown(data->ssl_handle);
+					break;
+
+				case SSL_ERROR_WANT_READ:
+				case SSL_ERROR_WANT_WRITE: {
+						php_pollfd p;
+						int i;
+
+						p.fd = ftp->fd;
+						p.events = (err == SSL_ERROR_WANT_READ) ? (POLLIN|POLLPRI) : POLLOUT;
+						p.revents = 0;
+
+						i = php_poll2(&p, 1, 300);
+
+						retry = i > 0;
+					}
+					break;
+
+				default:
+					php_error_docref(NULL, E_WARNING, "data_accept: SSL/TLS handshake failed");
+					SSL_shutdown(data->ssl_handle);
+					SSL_free(data->ssl_handle);
+					return 0;
+			}
+		} while (retry);
 
 		data->ssl_active = 1;
 	}
@@ -1575,16 +1734,19 @@ data_accepted:
 databuf_t*
 data_close(ftpbuf_t *ftp, databuf_t *data)
 {
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 	SSL_CTX		*ctx;
 #endif
 	if (data == NULL) {
 		return NULL;
 	}
 	if (data->listener != -1) {
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 		if (data->ssl_active) {
-			/* don't free the data context, it's the same as the control */
+
+			ctx = SSL_get_SSL_CTX(data->ssl_handle);
+			SSL_CTX_free(ctx);
+
 			SSL_shutdown(data->ssl_handle);
 			SSL_free(data->ssl_handle);
 			data->ssl_active = 0;
@@ -1593,9 +1755,11 @@ data_close(ftpbuf_t *ftp, databuf_t *data)
 		closesocket(data->listener);
 	}
 	if (data->fd != -1) {
-#if HAVE_OPENSSL_EXT
+#ifdef HAVE_FTP_SSL
 		if (data->ssl_active) {
-			/* don't free the data context, it's the same as the control */
+			ctx = SSL_get_SSL_CTX(data->ssl_handle);
+			SSL_CTX_free(ctx);
+
 			SSL_shutdown(data->ssl_handle);
 			SSL_free(data->ssl_handle);
 			data->ssl_active = 0;
@@ -1614,7 +1778,7 @@ data_close(ftpbuf_t *ftp, databuf_t *data)
 /* {{{ ftp_genlist
  */
 char**
-ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
+ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path)
 {
 	php_stream	*tmpstream = NULL;
 	databuf_t	*data = NULL;
@@ -1628,7 +1792,7 @@ ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
 
 
 	if ((tmpstream = php_stream_fopen_tmpfile()) == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to create temporary file.  Check permissions in temporary files directory.");
+		php_error_docref(NULL, E_WARNING, "Unable to create temporary file.  Check permissions in temporary files directory.");
 		return NULL;
 	}
 
@@ -1636,7 +1800,7 @@ ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
 		goto bail;
 	}
 
-	if ((data = ftp_getdata(ftp TSRMLS_CC)) == NULL) {
+	if ((data = ftp_getdata(ftp)) == NULL) {
 		goto bail;
 	}
 	ftp->data = data;
@@ -1656,7 +1820,7 @@ ftp_genlist(ftpbuf_t *ftp, const char *cmd, const char *path TSRMLS_DC)
 	}
 
 	/* pull data buffer into tmpfile */
-	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
+	if ((data = data_accept(data, ftp)) == NULL) {
 		goto bail;
 	}
 	size = 0;
@@ -1719,7 +1883,7 @@ bail:
 /* {{{ ftp_nb_get
  */
 int
-ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, long resumepos TSRMLS_DC)
+ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type, zend_long resumepos)
 {
 	databuf_t		*data = NULL;
 	char			arg[11];
@@ -1732,12 +1896,12 @@ ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t typ
 		goto bail;
 	}
 
-	if ((data = ftp_getdata(ftp TSRMLS_CC)) == NULL) {
+	if ((data = ftp_getdata(ftp)) == NULL) {
 		goto bail;
 	}
 
 	if (resumepos>0) {
-		snprintf(arg, sizeof(arg), "%ld", resumepos);
+		snprintf(arg, sizeof(arg), ZEND_LONG_FMT, resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1753,7 +1917,7 @@ ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t typ
 		goto bail;
 	}
 
-	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
+	if ((data = data_accept(data, ftp)) == NULL) {
 		goto bail;
 	}
 
@@ -1762,7 +1926,7 @@ ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t typ
 	ftp->lastch = 0;
 	ftp->nb = 1;
 
-	return (ftp_nb_continue_read(ftp TSRMLS_CC));
+	return (ftp_nb_continue_read(ftp));
 
 bail:
 	ftp->data = data_close(ftp, data);
@@ -1773,7 +1937,7 @@ bail:
 /* {{{ ftp_nb_continue_read
  */
 int
-ftp_nb_continue_read(ftpbuf_t *ftp TSRMLS_DC)
+ftp_nb_continue_read(ftpbuf_t *ftp)
 {
 	databuf_t	*data = NULL;
 	char		*ptr;
@@ -1836,7 +2000,7 @@ bail:
 /* {{{ ftp_nb_put
  */
 int
-ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, long startpos TSRMLS_DC)
+ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, zend_long startpos)
 {
 	databuf_t		*data = NULL;
 	char			arg[11];
@@ -1847,11 +2011,11 @@ ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type
 	if (!ftp_type(ftp, type)) {
 		goto bail;
 	}
-	if ((data = ftp_getdata(ftp TSRMLS_CC)) == NULL) {
+	if ((data = ftp_getdata(ftp)) == NULL) {
 		goto bail;
 	}
 	if (startpos > 0) {
-		snprintf(arg, sizeof(arg), "%ld", startpos);
+		snprintf(arg, sizeof(arg), ZEND_LONG_FMT, startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1866,7 +2030,7 @@ ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type
 	if (!ftp_getresp(ftp) || (ftp->resp != 150 && ftp->resp != 125)) {
 		goto bail;
 	}
-	if ((data = data_accept(data, ftp TSRMLS_CC)) == NULL) {
+	if ((data = data_accept(data, ftp)) == NULL) {
 		goto bail;
 	}
 	ftp->data = data;
@@ -1874,7 +2038,7 @@ ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type
 	ftp->lastch = 0;
 	ftp->nb = 1;
 
-	return (ftp_nb_continue_write(ftp TSRMLS_CC));
+	return (ftp_nb_continue_write(ftp));
 
 bail:
 	ftp->data = data_close(ftp, data);
@@ -1886,7 +2050,7 @@ bail:
 /* {{{ ftp_nb_continue_write
  */
 int
-ftp_nb_continue_write(ftpbuf_t *ftp TSRMLS_DC)
+ftp_nb_continue_write(ftpbuf_t *ftp)
 {
 	long			size;
 	char			*ptr;
