@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -32,39 +32,48 @@
 static void php_fsockopen_stream(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 {
 	char *host;
-	int host_len;
-	long port = -1;
+	size_t host_len;
+	zend_long port = -1;
 	zval *zerrno = NULL, *zerrstr = NULL;
-	double timeout = FG(default_socket_timeout);
-	unsigned long conv;
+	double timeout = (double)FG(default_socket_timeout);
+#ifndef PHP_WIN32
+	time_t conv;
+#else
+	long conv;
+#endif
 	struct timeval tv;
 	char *hashkey = NULL;
 	php_stream *stream = NULL;
 	int err;
 	char *hostname = NULL;
-	long hostname_len;
-	char *errstr = NULL;
+	size_t hostname_len;
+	zend_string *errstr = NULL;
 
 	RETVAL_FALSE;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lzzd", &host, &host_len, &port, &zerrno, &zerrstr, &timeout) == FAILURE) {
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|lz/z/d", &host, &host_len, &port, &zerrno, &zerrstr, &timeout) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	if (persistent) {
-		spprintf(&hashkey, 0, "pfsockopen__%s:%ld", host, port);
+		spprintf(&hashkey, 0, "pfsockopen__%s:" ZEND_LONG_FMT, host, port);
 	}
 
 	if (port > 0) {
-		hostname_len = spprintf(&hostname, 0, "%s:%ld", host, port);
+		hostname_len = spprintf(&hostname, 0, "%s:" ZEND_LONG_FMT, host, port);
 	} else {
 		hostname_len = host_len;
 		hostname = host;
 	}
-	
+
 	/* prepare the timeout value for use */
-	conv = (unsigned long) (timeout * 1000000.0);
+#ifndef PHP_WIN32
+	conv = (time_t) (timeout * 1000000.0);
 	tv.tv_sec = conv / 1000000;
+#else
+	conv = (long) (timeout * 1000000.0);
+	tv.tv_sec = conv / 1000000;
+#endif
 	tv.tv_usec = conv % 1000000;
 
 	if (zerrno)	{
@@ -73,7 +82,7 @@ static void php_fsockopen_stream(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 	}
 	if (zerrstr) {
 		zval_dtor(zerrstr);
-		ZVAL_STRING(zerrstr, "", 1);
+		ZVAL_EMPTY_STRING(zerrstr);
 	}
 
 	stream = php_stream_xport_create(hostname, hostname_len, REPORT_ERRORS,
@@ -83,13 +92,13 @@ static void php_fsockopen_stream(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 		efree(hostname);
 	}
 	if (stream == NULL) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to connect to %s:%ld (%s)", host, port, errstr == NULL ? "Unknown error" : errstr);
+		php_error_docref(NULL, E_WARNING, "unable to connect to %s:" ZEND_LONG_FMT " (%s)", host, port, errstr == NULL ? "Unknown error" : ZSTR_VAL(errstr));
 	}
 
 	if (hashkey) {
 		efree(hashkey);
 	}
-	
+
 	if (stream == NULL)	{
 		if (zerrno) {
 			zval_dtor(zerrno);
@@ -98,19 +107,18 @@ static void php_fsockopen_stream(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 		if (zerrstr && errstr) {
 			/* no need to dup; we need to efree buf anyway */
 			zval_dtor(zerrstr);
-			ZVAL_STRING(zerrstr, errstr, 0);
+			ZVAL_STR(zerrstr, errstr);
+		} else if (!zerrstr && errstr) {
+			zend_string_release(errstr);
 		}
-		else if (!zerrstr && errstr) {
-			efree(errstr);
-		} 
 
 		RETURN_FALSE;
 	}
 
 	if (errstr) {
-		efree(errstr);
+		zend_string_release(errstr);
 	}
-		
+
 	php_stream_to_zval(stream, return_value);
 }
 
@@ -123,6 +131,7 @@ PHP_FUNCTION(fsockopen)
 	php_fsockopen_stream(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
 /* }}} */
+
 /* {{{ proto resource pfsockopen(string hostname, int port [, int errno [, string errstr [, float timeout]]])
    Open persistent Internet or Unix domain socket connection */
 PHP_FUNCTION(pfsockopen)
